<?php

namespace Drupal\phpcs_tests\Form;

use Drupal\Core\Database\Connection;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\testsuite\BaseTrait;
use Drupal\phpcs_tests\PhpcsTestsFileResourceService;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Defines a form that configures forms module settings.
 */
class PhpcsTestsConfigurationForm extends ConfigFormBase
{
    use BaseTrait;

    /**
     * Initializer Service.
     *
     * @var \Drupal\phpcs_tests\PhpcsTestsFileResourceService
     */
    protected $fileResourceService;

    /**
     * The messenger service.
     *
     * @var Drupal\Core\Messenger\MessengerInterface
     */
    protected $messenger;

    /**
     * Path to contrib module directory.
     *
     * @var string
     */
    protected $contribModuleDirectory;

    /**
     * ConfigurationForm constructor.
     *
     * @param \Drupal\phpcs_tests\PhpcsTestsFileResourceService $fileResourceService
     *   The initializer object.
     * @param \Drupal\Core\Messenger\MessengerInterface         $messenger
     *   The messenger interface.
     */
    public function __construct(
        PhpcsTestsFileResourceService $fileResourceService,
        MessengerInterface $messenger,
    ) {
        $this->fileResourceService = $fileResourceService;
        $this->messenger = $messenger;
    }

    /**
     * {@inheritdoc}
     */
    public static function create(ContainerInterface $container)
    {
        return new static(
            $container->get('phpcs_tests.file_resource.service'),
            $container->get('messenger'),
        );
    }

    /**
     * {@inheritdoc}
     */
    public function getFormId()
    {
        return 'phpcs_tests_settings';
    }

    /**
     * {@inheritdoc}
     */
    protected function getEditableConfigNames()
    {
        return [
        'phpcs_tests.settings',
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function buildForm(array $form, FormStateInterface $form_state)
    {
        $config = $this->config('phpcs_tests.settings');

        $form['phpcs'] = [
        '#type' => 'details',
        '#title' => t('PHPCS settings'),
        '#description' => t('Setting related to creating the phpcs.xml file. PHP CodeSniffer is what the Drupal community uses to standardize the code created for custom modules. It works mainly with getting all developer to code in a certain style to make it easier to read.'),
        '#open' => true,
        ];

        $form['phpcs']['testsuite_phpcs_path'] = [
        '#type' => 'textfield',
        '#title' => $this->t('PHPCS File Path'),
        '#description' => $this->t("Folder to look in for files to run phpcs on."),
        '#default_value' => $config->get('testsuite_phpcs_path') ? $config->get('testsuite_phpcs_path') : '',
        ];

        return parent::buildForm($form, $form_state);
    }

    /**
     * {@inheritdoc}
     */
    public function validateForm(array &$form, FormStateInterface $form_state)
    {
        if ($form_state->hasValue('testsuite_phpcs_path') && $form_state->getValue('testsuite_phpcs_path') != "") {
            if (!preg_match('/^(.+)((\/|\\\)(.+))$/i', $form_state->getValue('testsuite_phpcs_path'))) {
                $form_state->setErrorByName(
                    'testsuite_phpcs_path',
                    $this->t('The phpcs path is invalid. Please enter a valid phpcs path.')
                );
            }
        }
    }

    /**
     * {@inheritdoc}
     */
    public function submitForm(array &$form, FormStateInterface $form_state)
    {
        $settings = $this->config('phpcs_tests.settings');
        $settings->set('testsuite_phpcs_path', $form_state->getValue('testsuite_phpcs_path'))
            ->save();

        $phpcsData['phpcs_path'] = $form_state->getValue('testsuite_phpcs_path');

        if ($this->fileResourceService->ifRootIsWritable()) {
            $this->fileResourceService->buildPhpcsXml($phpcsData);
        }
        else {
            $this->messenger->addStatus('The phpcs.xml file was unable to be created. This is most likely due to permissions on the root folder.');
        }

        parent::submitForm($form, $form_state);
    }

}
